{-|
Module      : Test.Data.Patch.Class
Description :
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

-}

module Test.Data.Patch.Class where

import Data.Patch.Class
import Data.Validity
import Test.Hspec


-- | Data.Patch.Class tests
spec_patch :: Spec
spec_patch = do
  describe "Patch" $ do
    it "works for simple patching" $ do
      let d = 3 :: Int
      let p = Replace 3 5 :: Replace Int

      act p d `shouldBe` 5

    it "works for simple validation" $ do
      let d = 3 :: Int
      let pInvalid = Replace 1 5 :: Replace Int
      let pValid = Replace 3 5 :: Replace Int

      validationIsValid (applicable pValid d) `shouldBe` True
      validationIsValid (applicable pInvalid d) `shouldBe` False

    it "works for simple composable" $ do
      let p = Replace {_old = 1, _new = 5} :: Replace Int
      let q = Replace {_old = 5, _new = 3} :: Replace Int

      -- We apply `q` on top of `p`
      validationIsValid (composable p q) `shouldBe` False
      validationIsValid (composable q p) `shouldBe` True

    it "works for simple transformWith (same _old, first has priority)" $ do
      -- | First patch always has a priority
      let conflictResolution a _b = a
      
      let p = Replace {_old = 1, _new = 3} :: Replace Int
      let q = Replace {_old = 1, _new = 5} :: Replace Int

      transformWith conflictResolution p q `shouldBe` (Replace {_old = 5, _new = 3}, Keep)

    it "works for simple transformWith (same _old, second has priority)" $ do
      -- | Second patch always has a priority
      let conflictResolution _a b = b
      
      let p = Replace {_old = 1, _new = 3} :: Replace Int
      let q = Replace {_old = 1, _new = 5} :: Replace Int

      transformWith conflictResolution p q `shouldBe` (Keep, Replace {_old = 3, _new = 5})

    it "works for more complex transformWith (different _old, first has priority)" $ do
      -- | First patch always has a priority
      let conflictResolution a _b = a
      
      let p = Replace {_old = 2, _new = 3} :: Replace Int
      let q = Replace {_old = 1, _new = 5} :: Replace Int

      transformWith conflictResolution p q `shouldBe` (Replace {_old = 5, _new = 3}, Keep)

    it "works for more complex transformWith (different _old, second has priority)" $ do
      -- | Second patch always has a priority
      let conflictResolution _a b = b
      
      let p = Replace {_old = 2, _new = 3} :: Replace Int
      let q = Replace {_old = 1, _new = 5} :: Replace Int

      transformWith conflictResolution p q `shouldBe` (Keep, Replace {_old = 3, _new = 5})

    it "can compute diff of 2 simple values" $ do
      let a = 3 :: Int
      let b = 5 :: Int
      let p = Replace { _old = a, _new = b } :: Replace Int
      diff a b `shouldBe` p
